/*======================================================
FILE:  bridle_GSM1xControl.c

SERVICES:  BRIDLE SWI Handler for GSM1xControl.

GENERAL DESCRIPTION:
   Implements the supervisor mode handler for BRIDLE GSM1xControl

PUBLIC CLASSES AND STATIC FUNCTIONS:

INITIALIZATION & SEQUENCING REQUIREMENTS:

   See Exported Routines

        Copyright  1999-2003 QUALCOMM Incorporated.
               All Rights Reserved.
            QUALCOMM Proprietary/GTDR
=====================================================*/
#include "BRIDLEInit.h"
#include "OEMFeatures.h"
#if defined(FEATURE_GSM1x)

#define _BRIDLE_GSM1XCONTROL_IMPL
#include "bridle_GSM1xControl.h"
#include "AEEBRIDLE.h"

#if __ARMCC_VERSION >= 120000
#pragma arm section zidata = "bridle_svc_rw_usr_ro"
#endif
uint32 bridle_oemGSM1xControl_swi;
#if __ARMCC_VERSION >= 120000
#pragma arm section zidata
#endif


/* List of handlers that patch calls to actual routines. The order 
** corresponds to the subroutine ids enumerated in the enum in bridle_GSM1xControl.h */
static const PFNSWIHANDLER gPtrTable[BRIDLE_GSM1XCONTROL_MAX] = { 
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_Init,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_ActivateNonGSM1xMode,         
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_GetSupportedProvisioningModes,        
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_GetUIMUniqueId,        
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_GetDFPresence,      
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_ProvisionGSM1xParameters,  
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_EnableGSM1xMode, 
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_GetPLMN,    
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_SetPLMN,   
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_GetGSM1xPRL,    
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_SetGSM1xPRL,       
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_ValidatePRL,        
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_SetGSM1xSIDNIDPairs,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_GetGSM1xSIDNIDPairs,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_GetAvailableModes,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_GetCurrentMode,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_ProcessPowerUp,
   (PFNSWIHANDLER)unpackbridle_OEMGSM1xControl_SVC_GetStatus
};

/*===========================================================================

FUNCTION   
   bridle_GSM1xControl_swihandler

DESCRIPTION
   Invokes the appropriate unpack routine and passes its arguments.
  
DEPENDENCIES
   This is invoked by SWI handler.

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
uint32 bridle_GSM1xControl_swihandler( uint32 arg1, uint32 arg2, uint32 arg3, uint32 arg4, uint32 arg5 )
{
   return bridle_generic_swihandler( arg1, arg2, arg3, arg4, arg5, 
                                     BRIDLE_GSM1XCONTROL_MAX, gPtrTable );
}

/*===========================================================================

FUNCTION   
   RegisterGSM1xControlForSWI

DESCRIPTION
   Registers with SWI handler and obtains ID for GSM1xControl Module.
  
DEPENDENCIES

RETURN VALUE

SIDE EFFECTS

===========================================================================*/
int RegisterGSM1xControlForSWI(void *po)
{
   return bridle_RegisterSubSystem( bridle_GSM1xControl_swihandler, &bridle_oemGSM1xControl_swi );
}
#endif // FEATURE_GSM1x
