#============================================================================
#  Name:
#    $(TARGET).MAK
#
#  Description:
#    Makefile to build the $(TARGET) downloadable module.
#
#   The following nmake targets are available in this makefile:
#
#     all           - make .elf and .mod image files (default)
#     clean         - delete object directory and image files
#     filename.o    - make object file
#
#   The above targets can be made with the following command:
#
#     nmake /f $(TARGET).mak [target]
#
#  Assumptions:
#    1. The environment variable ADSHOME is set to the root directory of the
#       arm tools.
#    2. The version of ADS is 1.2 or above.
#
#  Notes:
#    None.
#
#
#        Copyright  2000-2003 QUALCOMM Incorporated.
#               All Rights Reserved.
#            QUALCOMM Proprietary/GTDR
#
#----------------------------------------------------------------------------
#============================================================================
BREW_HOME      =$(BREWDIR)
ARM_HOME       =$(ARMHOME)
TARGET         =C:\哈\t~1\90_~1\90_~1\src\iautolog\iautolog
OBJS           =AEEAPP~1.o AEEMOD~1.o errorlib.o iautolog.o locallog.o 
APP_INCLUDES   =

#-------------------------------------------------------------------------------
# Target file name and type definitions
#-------------------------------------------------------------------------------

EXETYPE    = elf                # Target image file format
MODULE     = mod                # Downloadable module extension

#-------------------------------------------------------------------------------
# Target compile time symbol definitions
#
# Tells the SDK source stuffs that we're building a dynamic app.
#-------------------------------------------------------------------------------

DYNAPP          = -DDYNAMIC_APP


#-------------------------------------------------------------------------------
# Software tool and environment definitions
#-------------------------------------------------------------------------------

AEESRCPATH = $(BREW_HOME)\src
AEEINCPATH = $(BREW_HOME)\inc

ARMBIN = $(ARM_HOME)\bin        # ARM ADS application directory
ARMINC = $(ARM_HOME)\include    # ARM ADS include file directory
ARMLIB = $(ARM_HOME)\lib        # ARM ADS library directory

ARMCC   = $(ARMBIN)\armcc       # ARM ADS ARM 32-bit inst. set ANSI C compiler
LD      = $(ARMBIN)\armlink     # ARM ADS linker
HEXTOOL = $(ARMBIN)\fromelf     # ARM ADS utility to create hex file from image

OBJ_CMD    = -o                 # Command line option to specify output filename

#-------------------------------------------------------------------------------
# Processor architecture options
#-------------------------------------------------------------------------------

CPU = -cpu ARM7TDMI             # ARM7TDMI target processor

#-------------------------------------------------------------------------------
# ARM Procedure Call Standard (APCS) options
#-------------------------------------------------------------------------------

ROPI     = ropi                 # Read-Only(code) Position independence
INTERWRK = interwork            # Allow ARM-Thumb interworking

APCS = -apcs /$(ROPI)/$(INTERWRK)/norwpi

#-------------------------------------------------------------------------------
# Additional compile time error checking options
#-------------------------------------------------------------------------------

CHK = -fa                       # Check for data flow anomolies

#-------------------------------------------------------------------------------
# Compiler output options
#-------------------------------------------------------------------------------

OUT = -c                        # Object file output only

#-------------------------------------------------------------------------------
# Compiler/assembler debug options
#-------------------------------------------------------------------------------

DBG = -g                        # Enable debug

#-------------------------------------------------------------------------------
# Compiler optimization options
#-------------------------------------------------------------------------------

OPT = -Ospace -O2               # Full compiler optimization for space

#-------------------------------------------------------------------------------
# Compiler code generation options
#-------------------------------------------------------------------------------

END = -littleend                # Compile for little endian memory architecture
ZA  = -zo                       # LDR may only access 32-bit aligned addresses

CODE = $(END) $(ZA)


#-------------------------------------------------------------------------------
# Include file search path options
#-------------------------------------------------------------------------------

INC = -I. -I$(AEEINCPATH) $(APP_INCLUDES)


#-------------------------------------------------------------------------------
# Compiler pragma emulation options
#-------------------------------------------------------------------------------


#-------------------------------------------------------------------------------
# Linker options
#-------------------------------------------------------------------------------

LINK_CMD = -o                    #Command line option to specify output file
                                 #on linking

ROPILINK = -ropi                 #Link image as Read-Only Position Independent

LINK_ORDER = -first AEEMod_Load

#-------------------------------------------------------------------------------
# HEXTOOL options
#-------------------------------------------------------------------------------

BINFORMAT = -bin


#-------------------------------------------------------------------------------
# Compiler flag definitions
#-------------------------------------------------------------------------------

CFLAGS0 = $(OUT) $(DYNAPP) $(CPU) $(APCS) $(CODE) $(CHK) $(DBG)
CFLAGS  = $(CFLAGS0) $(OPT)

#-------------------------------------------------------------------------------
# Linker flag definitions
#-------------------------------------------------------------------------------

# the -entry flag is not really needed, but it keeps the linker from reporting
# warning L6305W (no entry point).  The address
LFLAGS = $(ROPILINK) -rwpi -entry 0x8000#

#----------------------------------------------------------------------------
# Default target
#----------------------------------------------------------------------------

all :  $(TARGET).$(MODULE)

#----------------------------------------------------------------------------
# Clean target
#----------------------------------------------------------------------------

# The object subdirectory, target image file, and target hex file are deleted.

clean :
        @echo ---------------------------------------------------------------
        @echo CLEAN
        -del /f $(OBJS)
        -del /f $(TARGET).$(EXETYPE)
        -del /f $(TARGET).$(MODULE)
        @echo ---------------------------------------------------------------

#============================================================================
#                           DEFAULT SUFFIX RULES
#============================================================================

# The following are the default suffix rules used to compile all objects that
# are not specifically included in one of the module specific rules defined
# in the next section.

# The following macros are used to specify the output object file, MSG_FILE
# symbol definition and input source file on the compile line in the rules
# defined below.

SRC_FILE = $(@F:.o=.c)                  # Input source file specification
OBJ_FILE = $(OBJ_CMD) $(@F)   # Output object file specification

.SUFFIXES :
.SUFFIXES : .o .dep .c

#--------------------------------------------------------------------------
# C code inference rules
#----------------------------------------------------------------------------

.c.o:
        @echo ---------------------------------------------------------------
        @echo OBJECT $(@F)
        $(ARMCC) $(CFLAGS) $(INC) $(OBJ_FILE) $(SRC_FILE)
        @echo ---------------------------------------------------------------

.c.mix:
        @echo ---------------------------------------------------------------
        @echo OBJECT $(@F)
        $(ARMCC) -S -fs $(CFLAGS) $(INC) $(OBJ_FILE) $<
        @echo ---------------------------------------------------------------


{$(AEESRCPATH)}.c.o:
        @echo ---------------------------------------------------------------
        @echo OBJECT $(@F)
        $(ARMCC) $(CFLAGS) $(INC) $(OBJ_FILE) $(AEESRCPATH)\$(SRC_FILE)
        @echo ---------------------------------------------------------------


#===============================================================================
#                           MODULE SPECIFIC RULES
#===============================================================================

APP_OBJS = $(OBJS)


#----------------------------------------------------------------------------
# Lib file targets
#----------------------------------------------------------------------------

$(TARGET).$(MODULE) : $(TARGET).$(EXETYPE)
        @echo ---------------------------------------------------------------
        @echo TARGET $@
        $(HEXTOOL)  $(TARGET).$(EXETYPE) $(BINFORMAT) $(TARGET).$(MODULE)

$(TARGET).$(EXETYPE) : $(APP_OBJS)
        @echo ---------------------------------------------------------------
        @echo TARGET $@
        $(LD) $(LINK_CMD) $(TARGET).$(EXETYPE) $(LFLAGS) $(APP_OBJS) $(LINK_ORDER)

#----------------------------------------------------------------------------
# Applet Specific Rules
#----------------------------------------------------------------------------


RULE1 = c:\PROGRA~1\BREW31~1.2JA\sdk\src
{$(RULE1)}.c.o:
	@echo ---------------------------------------------------------------
	@echo OBJECT $(@F)
	$(ARMCC) $(CFLAGS) $(INC) $(OBJ_FILE)  $(RULE1)\$(SRC_FILE)
	@echo ---------------------------------------------------------------


RULE2 = c:\哈\t~1\90_~1\90_~1\src\iautolog
{$(RULE2)}.c.o:
	@echo ---------------------------------------------------------------
	@echo OBJECT $(@F)
	$(ARMCC) $(CFLAGS) $(INC) $(OBJ_FILE)  $(RULE2)\$(SRC_FILE)
	@echo ---------------------------------------------------------------


# --------------------------------------------
# DEPENDENCY LIST, DO NOT EDIT BELOW THIS LINE
# --------------------------------------------


AEEAPP~1.o:	c:\PROGRA~1\BREW31~1.2JA\sdk\src\AEEAPP~1.C
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEGroupIDs.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEClassIds.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEInterface.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEQueryInterface.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEInterface.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEVCodes.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEDisp.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEBitmap.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEFont.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEBitmap.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEImage.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESoundPlayer.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESound.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEResourceCtl.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEControls_res.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEELngCode.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEStdLib.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEAppGen.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEModGen.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEModGen.h
AEEAPP~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\nmdef.h
AEEMOD~1.o:	c:\PROGRA~1\BREW31~1.2JA\sdk\src\AEEMOD~1.C
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEHeap.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEGroupIDs.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEClassIds.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEInterface.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEQueryInterface.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEInterface.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEStdLib.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEStdLib.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEModGen.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEVCodes.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEDisp.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEBitmap.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEFont.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEBitmap.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEImage.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESoundPlayer.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESound.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEResourceCtl.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEControls_res.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEELngCode.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEAppGen.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEModGen.h
AEEMOD~1.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\nmdef.h
errorlib.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\errorlib.c
errorlib.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\iautolog.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEGroupIDs.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEClassIds.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEInterface.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEQueryInterface.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEInterface.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEModGen.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEVCodes.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEDisp.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEBitmap.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEFont.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEBitmap.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEImage.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESoundPlayer.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESound.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEResourceCtl.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEControls_res.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEELngCode.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEAppGen.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEModGen.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEHtmlViewer.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEWeb.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESource.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEENet.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\NET_URGENT.bid
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEWebOpts.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEFile.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEHeap.h 
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEStdLib.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEStdLib.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
errorlib.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\KDDIError.h
errorlib.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\iautolog.bid
iautolog.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\iautolog.c
iautolog.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\iautolog.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEGroupIDs.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEClassIds.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEInterface.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEQueryInterface.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEInterface.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEModGen.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEVCodes.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEDisp.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEBitmap.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEFont.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEBitmap.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEImage.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESoundPlayer.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESound.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEResourceCtl.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEControls_res.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEELngCode.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEAppGen.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEModGen.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEHtmlViewer.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEWeb.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESource.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEENet.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\NET_URGENT.bid
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEWebOpts.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEFile.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEHeap.h 
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEStdLib.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEStdLib.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
iautolog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\KDDIError.h
iautolog.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\iautolog.bid
iautolog.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\errorlib.c
iautolog.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\iautolog.h
locallog.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\locallog.c
locallog.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\iautolog.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEGroupIDs.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEClassIds.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEInterface.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEQueryInterface.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEInterface.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEModGen.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEVCodes.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEDisp.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEBitmap.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEFont.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEBitmap.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEImage.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESoundPlayer.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESound.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEResourceCtl.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEControls_res.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEELngCode.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEAppGen.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEModGen.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEHtmlViewer.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEWeb.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEESource.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEENet.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEShell.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\NET_URGENT.bid
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEWebOpts.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEFile.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEHeap.h 
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEStdLib.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEComdef.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEE.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEStdLib.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\AEEError.h
locallog.o:	C:\PROGRA~1\BREW31~1.2\SDK\inc\KDDIError.h
locallog.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\iautolog.bid
locallog.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\errorlib.c
locallog.o:	c:\哈\t~1\90_~1\90_~1\src\iautolog\iautolog.h
